ASMB,A,L,B,T
      ORG 37213B
* 
********** SYSTEM INPUT/OUTPUT DRIVER *************** 
*  FOR THE 300 CPS PAPER TAPE READER  * 
* 
* THE DRIVER PROVIDES THE INPUT FACILITY REQUIRED 
*  BY THE ASSEMBLER, SYMBOLIC EDITOR AND FORTRAN
*  COMPILER.  THE LINKAGE ADDRESS FOR THIS FACILITY 
*  IS AT LOCATION 101B IN THE SYSTEM LINKAGE TABLE. 
* 
* CALLING SEQUENCE: 
* 
*  (A) = LENGTH OF DATA BUFFER
*       > 0, LENGTH IN CHARACTERS FOR ASCII RECORD
*       < 0, LENGTH IN WORDS (2'S COMPLEMENT) FOR 
*             BINARY RECORD 
* 
*  (B) = STARTING ADDRESS OF DATA BUFFER
* 
*  (P)       JSB   101B,I 
*  (P+1)    -RETURN-  (B) MEANINGLESS 
* 
*    (A) ON RETURN CONTAINS THE NUMBER OF CHARACTERS
*        (ASCII RECORD) OR WORDS (BINARY RECORD)
*        TRANSMITTED TO BUFFER
* 
*     NOTE* IF A = 0 ON RETURN, AN END-OF-TAPE
*           CONDITION RESULTED FROM THE INPUT.
*           THIS IS DEFINED AS A FIXED NUMBER OF
*           CONSECUTIVE FEED FRAMES INPUT BEFORE
*           THE FIRST NON-ZERO DATA CHARACTER OF
*           A RECORD. THE FIXED NUMBER IS THE 
*           NEGATIVE CONSTANT AT LABEL -FFCT-.
* 
* RECORD FORMATS- 
* 
*   ASCII * A GROUP OF CHARACTERS TERMINATED BY A 
*           RETURN/LINE FEED. THESE CODES INPUT 
*           BEFORE THE FIRST DATA CHARACTER OF A
*           RECORD ARE IGNORED. A RUB-OUT CODE
*           CAUSES THE CURRENT RECORD TO BE DELETED 
*           AND THE NEXT RECORD INPUT. A LINE-FEED
*           MUST TERMINATE THE CURRENT RECORD.
* 
*   BINARY* THE FIRST CHARACTER OF A BINARY RECORD
*           CONTAINS THE WORD COUNT OF THE ENTIRE 
*           RECORD. THIS VALUE DETERMINES THE NUMBER
*           OF WORDS (UP TO THE BUFFER LENGTH 
*           REQUESTED) TO BE TRANSMITTED
* 
* 
*****  DRIVER INITIALIZATION SECTION  ***** 
* 
* PURPOSE OF THIS SECTION IS TO SET THE I/O 
*  INSTRUCTIONS TO REFERENCE A DESIGNATED I/O 
*  ADDRESS FOR THE 2737A TAPE READER. 
* 
* EXECUTION OF THIS SECTION IS BEGUN AT LOCATION 2: 
*      (2)  JMP  3,I
*      (3)  DEF  I.PTR
*  WITH SWITCHES 05-00 = I/O ADDRESS. 
* 
* THIS SECTION IS RELEASED AFTER THE
*  INITIALIZATION IS COMPLETED. 
* 
      ORG 2 
* 
      JMP 3,I      SET STARTING POINT 
      DEF I.PTR     IN 2 AND 3. 
      ORR 
* 
I.PTR LIA 01       GET AND
      AND M.77      ISOLATE I/O ADDRESS OF READER 
      STA 1        SET IN B.
      LDA PT.2     -OR- 
      IOR 1         I/O 
      STA PT.2      ADDRESS 
      LDA PT.2+1    IN
      IOR 1         I/O 
      STA PT.2+1    INSTRUCTIONS
      LDA PT.2+3
      IOR 1 
      STA PT.2+3
      LDA PT.11 
      IOR 1 
      STA PT.11 
* 
      CCA          (A) = 177777 
      HLT 77B       -HALT,
      JMP *-1        IRRECOVERABLE. 
* 
M.77  OCT 77       I/O ADDRESS MASK.
* 
LWAM  EQU M.77     DEFINE LWA AVAILABLE MEMORY. 
* 
* 
PT.R  NOP           * ENTRY / EXIT POINT *
      STB BUFF     SET BUFFER 
      STB BUFS      ADDRESS 
      CLB           SET DATA-IN FLAG TO SAY NO DATA 
      CCE,SSA      SET E=1 FOR ASCII INPUT
      ALS,CLE,SLA  SET E=0 FOR BINARY-CONVERT TO
      CMA,INA       CHARS, SET CHAR CNT NEGATIVE
      STA LENG       FOR INDEX. 
PT.1  STA CHC1     SET CHARACTER INDEX
      LDA UL52     INITIALIZE UPPER/LOWER CHARACTER 
      STA ULFL      POSITION INDICATOR TO UPPER 
      LDA FFCT     INITIALIZE FEED-FRAME COUNTER
      STA EOTC      FOR  -END OF TAPE-
      CLA          SET CHARACTER INPUT
      STA CHC2      COUNTER = 0.
* 
* CHARACTER INPUT SECTION 
* 
PT.2  STC INPUT,C  SET DEVICE TO READ ONE CHARACTER 
      SFS INPUT    WAIT FOR CHARACTER TO
      JMP *-1        BE INPUT  (FLAG = 1) 
      LIA INPUT    GET CHARACTER FROM BUFFER
      SEZ          IF ASCII MODE, 
      AND M177      REMOVE 8-LEVEL BIT
      SZB,RSS      IF NO DATA YET AND CHAR. = 0,
      SZA           CHECK FOR END-OF-TAPE.
      JMP PT.3     OTHERWISE, PROCESS CHARACTER 
* 
      ISZ EOTC     INDEX END-OF-TAPE COUNTER
      JMP PT.2      NO -
      JMP PT.10    END-OF-TAPE, GO TO EXIT SECTION. 
* 
* BINARY RECORD LENGTH SECTION
* 
PT.3  SEZ          IF ASCII MODE, GO TO EXAMINE 
      JMP PT.4      CHARACTER 
      SZB          IF NOT FIRST CHAR. OF BINARY 
      JMP PT.5      RECORD, GO TO PACKING SECTION.
* 
      STA 1        CONVERT RECORD WORD LENGTH TO
      BLS           CHARACTERS
      ADB LENG     SUBTRACT ORIGINAL BUFFER LENGTH. 
      CLE,SSB,RSS  IF RECORD LENGTH GT BUFFER 
      JMP PT.5      LENGTH, USE BUFFER LENGTH.
      STA 1        CONVERT LENGTH TO
      BLS           CHARACTERS, SET NEGATIVE
      CMB,INB       AND STORE AS
      STB CHC1       CHARACTER INDEX. 
      JMP PT.5     GO TO PACK CHARACTER.
* 
* ASCII CHARACTER EXAMINATION SECTION 
* 
PT.4  CPA RETN     IF CHAR. IS A RETURN,
      JMP PT.2      IGNORE IT.
      CPA RUBO     IF CHAR. IS A RUBOUT,
      JMP PT.8      GO TO INPUT NEXT RECORD 
      CPA LINF     IF CHAR. IS A LINE-FEED, GO TO 
      JMP PT.9      TERMINATION SECTION.
* 
      SSB          IF A LINE-FEED IS NEEDED, DATA-IN
      JMP PT.2      FLAG = -1, GO INPUT NEXT CHAR.
* 
PT.5  CLB,INB      SET DATA-IN FLAG TO SAY RECORD 
      STB SAVB      BEING INPUT  (=1) 
* 
* DATA PACKING AND STORING SECTION
* 
      LDB ULFL     (B) = CHAR. POSITION INDICATOR 
      SSB,RSS 
      JMP PT.6     -UPPER CHAR. 
      IOR CHAR     LOWER- PACK IN PREVIOUS UPPER
      STA BUFF,I    CHAR. AND STORE IN BUFFER 
      ISZ BUFF     ADD 1 TO BUFFER ADDRESS. 
      JMP PT.7
PT.6  ALF,ALF     ROTATE CHAR. TO UPPER POSITION
      STA CHAR      AND SAVE IT.
PT.7  RBL          SET INDICATOR FOR NEXT 
      STB ULFL      POSITION
      LDB SAVB     RESTORE DATA-IN FLAG.
      ISZ CHC2     ADD 1 TO CHAR. INPUT COUNTER 
      ISZ CHC1     INDEX CHARACTER INPUT COUNTER. 
      JMP PT.2     BUFFER NOT FILLED. 
      SEZ,RSS      BUFFER FILLED, IF BINARY MODE, 
      JMP PT.10     GO TO EXIT SECTION. 
* 
      CCB          SET DATA-IN FLAG =-1 TO FORCE A
      JMP PT.2      LINE-FEED TO BE INPUT 
* 
* SECTION TO PROCESS RUBOUT CODE (ASCII MODE) 
* 
PT.8  LDA BUFS     RESTORE STARTING BUFFER
      STA BUFF      ADDRESS 
      LDA LENG     (A) = ORIGINAL BUFFER LENGTH 
      CCB          SET DATA-IN FLAG = -1 TO FORCE A 
      JMP PT.1      LINE-FEED TO BE INPUT.
* 
* SECTION TO PROCESS LINE-FEED CODE (ASCII MODE)
* 
PT.9  CLB         SET B=0 FOR POSSIBLE RE-INPUT FROM
      LDA LENG     RUBOUT. IF NO CHARS INPUT OR 
      CPA CHC1      RUBOUT RECORD TERMINATED ,
      JMP PT.2       READ NEXT CHARACTER. 
* 
      LDB ULFL     IF LAST CHARACTER
      LDA CHAR      INPUT WAS IN
      IOR BLANK      UPPER POSITION, MAKE 
      SSB             LOWER CHAR A BLANK
      STA BUFF,I       AND STORE LAST WORD. 
* 
* EXIT (COMPLETION) SECTION 
* 
PT.10 LDA CHC2     (A) = NUMBER OF CHARS INPUT. 
      SEZ,RSS      IF BINARY RECORD,
      ARS           CONVERT TO WORDS
PT.11 CLC INPUT    CLEAR DEVICE CONTROL.
      JMP PT.R,I   EXIT.
* 
* CONSTANT, STORAGE AND FLAG SECTION
* 
BUFF  OCT 0        HOLDS CURRENT BUFFER ADDRESS 
BUFS  OCT 0        HOLDS STARTING BUFFER ADDRESS
LENG  OCT 0        HOLDS BUFFER LENGTH
CHC1  OCT 0        CHARACTER INDEX
CHC2  OCT 0        CHARACTER INPUT COUNTER
CHAR  OCT 0        TEMPORARY STORAGE OF UPPER CHAR. 
BLANK OCT 40       ASCII BLANK (SPACE)
SAVB  OCT 0        TEMPORARY STORAGE
RETN  OCT 15       ASCII RETURN CODE
RUBO  OCT 177      ASCII RUBOUT CODE
LINF  OCT 12       ASCII LINE-FEED CODE 
M177  EQU RUBO     7-BIT MASK 
UL52  OCT 52525    UPPER/LOWER CHAR POSITION
ULFL  OCT 0         INDICATOR. 0=UPPER, 1=LOWER.
EOTC  EQU CHAR
* 
*  FOLLOWING IS NEGATIVE CONSTANT FOR FEED-FRAME COUNT
* 
FFCT  DEC -10      * END OF TAPE COUNTER *
* 
INPUT EQU 0 
* 
* SYSTEM LINKAGE TABLE SET-UP 
* 
      ORG 101B
      ABS PT.R     DRIVER ADDRESS 
      ORG 106B
      ABS LWAM     SETS LWA OF AVAILABLE MEMORY 
* 
      END 
